class BibTeX::Parser

rule

  file            : opt_space                         { result = val[0]    }
                  | opt_space object_list opt_space   { result = val[1]    }
                  ;

  object_list     : object                            { result = [val[0]]  }
                  | object_list opt_space object      { result << val[2]   }
	          ;

  object          : AT opt_space at_object            { result = val[2]    }
                  ;

  at_object       : comment                           { result = val[0]     }
                  | entry                             { result = val[0]     }
                  ;

  comment         : COMMENT opt_space value           { result = [:comment, val[2]] } 
                  ;
		  
  entry           : entry_head assignment_list
                      RBRACE                          { val[0].properties = val[1]; result = val[0] }
		  | entry_head assignment_list        /* Allow additional comma after last entry */
                      COMMA opt_space RBRACE          { val[0].properties = val[1]; result = val[0] }
                  | entry_head RBRACE                 { result = val[0] } /* Empty entry */
		  ;

  entry_head      : entry_name opt_space
                        LBRACE opt_space
                        key_name opt_space
                        COMMA opt_space               { result = Entry.new(val[0], val[4]) }
                  ;

  entry_name      : WORD                           
                  ;

  key_name        :                                   { result = "" }
                  | CHARACTER 
                  | WORD                            
                  | key_name CHARACTER                { result << val[1]  }
                  | key_name WORD                     { result << val[1]  }
                  ;

  assignment_list : assignment                        { result = val[0] }
                  | assignment_list COMMA opt_space
                        assignment                    { result.merge!(val[3]) }
                  ;

  assignment      : assignment_lhs opt_space
                    EQUALS opt_space              
                        value opt_space               { result = {val[0].downcase.to_sym => val[4]} }
                  ;

  assignment_lhs  : WORD
                  ;

  value           : LBRACE      value_content RBRACE             { result = val[1] }
                  | SINGLE_ANFZ value_content_single_del SINGLE_ANFZ   { result = val[1] }
		  | DOUBLE_ANFZ value_content_double_del DOUBLE_ANFZ   { result = val[1] }
		  | WORD
		  ;

  value_content_single_del   :                       { result = "" }
                  | value_content SPACE              { result << val[1] }
                  | value_content WORD               { result << val[1] }
                  | value_content COMMA              { result << val[1] }
/*                  | value_content "\\" SINGLE_ANFZ   { result << val[1] << val[2]} */
                  | value_content DOUBLE_ANFZ        { result << val[1] }
                  | value_content CHARACTER          { result << val[1] }
                  | value_content AT                 { result << val[1] }
                  | value_content LBRACE             { result << val[1] }
                  | value_content RBRACE             { result << val[1] }
		  ;

  value_content_double_del   :                       { result = "" }
                  | value_content SPACE              { result << val[1] }
                  | value_content WORD               { result << val[1] }
                  | value_content COMMA              { result << val[1] }
                  | value_content SINGLE_ANFZ        { result << val[1] }
/*                  | value_content "\\" DOUBLE_ANFZ   { result << val[1] << val[2] } */
                  | value_content CHARACTER          { result << val[1] }
                  | value_content AT                 { result << val[1] }
                  | value_content LBRACE             { result << val[1] }
                  | value_content RBRACE             { result << val[1] }
		  ;

  value_content   :                                   { result = "" }
                  | value_content SPACE              { result << val[1] }
                  | value_content WORD               { result << val[1] }
                  | value_content COMMA              { result << val[1] }
                  | value_content SINGLE_ANFZ        { result << val[1] }
                  | value_content DOUBLE_ANFZ        { result << val[1] }
                  | value_content CHARACTER          { result << val[1] }
                  | value_content AT                 { result << val[1] }
                  | value_content 
		         LBRACE value_content RBRACE { result << val[1] << val[2] << val[3] }
		  ;

  opt_space       : 
                  | SPACE                             
	          ;

end


---- header ----

module BibTeX
  Entry = Struct.new(:type, :key, :properties)
end

---- inner ----

  def report(*args)
    p args
  end
  
  def parse( str )
    @q = []

    @yydebug = true

    while str.size > 0 do
      case str
      when /\A[\t\n\s]+/o
        @q.push [:SPACE, $&]
      when /\ACOMMENT/o
        @q.push [:COMMENT, $&]
      when /\A\,/o
        @q.push [:COMMA, $&]
      when /\A@/o
        @q.push [:AT, $&]
      when /\A\{/o
        @q.push [:LBRACE, $&]
      when /\A\}/o
        @q.push [:RBRACE, $&]
      when /\A\=/o
        @q.push [:EQUALS, $&]
      when /\A\'/o
        @q.push [:SINGLE_ANFZ, $&]
      when /\A\"/o
        @q.push [:DOUBLE_ANFZ, $&]
      when /\A\w+/o
        @q.push [:WORD, $&]
      when /\A./o
        @q.push [:CHARACTER, $&]
      when /\A\d+/o
        @q.push [:NUMBER, $&.to_i]
      end
      str = $'
    end
    @q.push [false, '$end']

    do_parse
  end

  def next_token
    @q.shift
  end


---- footer ----
