require 'googlescholar'
require 'net/http'
require 'rdf/redland'
require 'uri'

# WISHLIST: Cluster IDs for single-paper clusters
# WISHLIST: Easy search (haha!)
# WISHLIST: 'Subject Area' information

# Get the command line arguments from ARGV and make into query string
# Use to search Google Scholar
# Permute 'grouped' searches
args = []
ARGV.each do | arg |
  arg = arg.split ' '
  if arg.size > 1
    arg[0] = '"' + arg[0]
    arg[arg.size - 1] = arg[arg.size - 1] + '"'
  end
  arg.each do | newarg |
    args.push newarg
  end
end
query = args.join '+'
query = URI.escape(query)

# Here are hashes of the codes for school indices and bibliographic formats.
schools = {
  :harvard => '50d1d418caa3f85e',
  :oclc    => '7e6cc990821af63',
  :rpi     => '782749c87e657d9c'
}

bibformats = {
  :endnote => 3,
  :bibtex  => 4
}

# Set up cookie for search to get BibTeX, WorldCat, and Harvard links
cookie = "IN=#{schools[:oclc]}+#{schools[:harvard]}:CF=#{bibformats[:bibtex]}"

# Get the first page of results for search and render it into RDF.
resp = ''
begin
  http = Net::HTTP.new 'scholar.google.com', 80
  
  # Get the homepage to get a user ID
  # TODO: Save cookie
  http.start do | http |
    req = Net::HTTP::Get.new '/'
    response = http.request req
    setcookie = response['Set-Cookie'].split ', '
    id = ''
    # Get the GSP cookie...
    setcookie.each do | c |
      if c.include? 'GSP='
        id = c.split('; ')[0]
        id = id.slice 4, id.length
        break
      end
    end
    # And get the ID value out of the GSP cookie
    if id != ''
      id = id.split ':'
      id.each do | pair |
        pair = pair.split '='
        if pair[0] == 'ID'
          id = pair[1]
          break
        end
      end
    end
    cookie = "ID=#{id}:#{cookie}"
  end

  # Now do the actual search
  http.start do | http |
    req = Net::HTTP::Get.new "/scholar?q=#{query}&hl=en&lr=&btnG=Search",
        {'Cookie', "GSP=#{cookie}"}
    response = http.request req
    resp = response.body
  end

  # FINALLY...  We get to actually parse the result into RDF
  model = GoogleScholar.parse resp, cookie
  
  serializer = Redland::Serializer.new name='rdfxml-abbrev'
  serializer.set_namespace 'bibo', 'http://purl.org/ontology/bibo/'
  serializer.set_namespace 'foaf', 'http://xmlns.com/foaf/0.1/'
  serializer.set_namespace 'bibo_ext', 'http://www.pipian.com/schemas/bibo-ext/'
  serializer.set_namespace 'dcterms', 'http://purl.org/dc/terms/'
  
  serializer.to_file 'scholar.rdf', model

rescue SocketError
  raise 'Could not connect to host %s' % host
end